function output_data = RAAR(input_data)
%%% Input
% input_data structure
%%% Output 
% output_data structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% calculate initial reflector of initialGuess over magnitude constraint
x    = input_data.initialGuess;
tmp1 = 2*magnitudeProjection( input_data, x ) - x;    
k = 1;
output_data.error = zeros(input_data.algorithm.maxit);
error = inf;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
while ( k < input_data.algorithm.maxit && error > input_data.algorithm.error )
  
    % compute relaxation parameter beta_k
    beta  = computeRelaxationParameter(input_data, k);  
    gamma = computeShrinkageParameter(input_data, k);
    
    if isfield(input_data.problem.constraint, 'info')
        itMsg = sprintf('Iteration: %d, Gamma: %.6f, Beta: %.3f, Error: %.5f, Constraint: %s, Type: %s', k, gamma, beta, error, input_data.problem.constraint.name, input_data.problem.constraint.info);
    else
        itMsg = sprintf('Iteration: %d, Gamma: %.6f, Beta: %.3f, Error: %.3f, Constraint: %s', k, gamma, beta, error, input_data.problem.constraint.name);
    end
    disp(itMsg);
    
    tmp3 = constraintProximityOperator(input_data, tmp1, gamma);
    
    x_old = x;
    tmp_u = (beta*(2*tmp3-tmp1) + (1-beta)*tmp1 + x)/2;     % update rule RAAR
    x     = tmp_u;                                          
    tmp1  = 2*magnitudeProjection( input_data, x ) - x;     % calculate reflector for next iteration   
    
    %if strcmp(input_data.algorithm.mode, 'verbose')
    %    output_data.iterates(k) = u;
    %end
    
    error = computeError(input_data, x, x_old, tmp3);
    output_data.error(k) = error;

    k = k+1;
end

output_data.error = output_data.error(1:k-1);
output_data.reconstruction = x;
output_data.problem = input_data.problem;
output_data.algorithm = input_data.algorithm;
if isfield(input_data, 'physicalParameters')
    output_data.physicalParameters = input_data.physicalParameters;
end
%output_data.error = error;