% Implements range constraints (positivity, negativitiy, amplitude <= 1)
% for different types of objects.

function outputPsi = proxRangeConstraint (input_data, Psi)

if strcmp(input_data.problem.type, 'phase')

    PsiPhase         = angle(Psi);
    
    % if no range type is specified, we go with negativity for phase
    % objects.
    if isfield(input_data.problem.constraint, 'range')
        if strcmp(input_data.problem.constraint.range, 'positive')
            outputPsiPhase = max(PsiPhase, 0);
        else        
            outputPsiPhase = min(PsiPhase, 0);
        end
    else
            outputPsiPhase = min(PsiPhase, 0);
    end
    
    outputPsi        = exp(1i*outputPsiPhase);
    
elseif strcmp(input_data.problem.type, 'amplitude')
    
    PsiAmplitude     = abs(Psi);
    outputPsi        = min(PsiAmplitude, 1);

elseif strcmp(input_data.problem.type, 'mixed')          
    
    PsiPhase         = angle(Psi);
    PsiAmplitude     = abs(Psi);
    
    outputPsiPhase   = min(PsiPhase, 0);
    outputPsiAmplitude = min(PsiAmplitude, 1);
    
    outputPsi        = outputPsiAmplitude .* exp(1i*outputPsiPhase);
    
elseif strcmp(input_data.problem.type, 'real')
    
    outputPsi        = max(real(Psi), 0);
    
else error('No suitable problem type specified in input_data.problem.type.')

end