% Implements support constraints for different objects.

function outputPsi = proxSupportConstraint (input_data, Psi)

if strcmp(input_data.problem.type, 'phase') %|| strcmp(input_data.problem.type, 'amplitude')
    
    PsiPhase         = angle(Psi);
    outputSupport    = input_data.problem.constraint.support;
    
    outputPsiPhase   = zeros(size(Psi));  
    outputPsiPhase(outputSupport) = PsiPhase(outputSupport);
    
    outputPsi        = exp(1i*outputPsiPhase);
    
elseif strcmp(input_data.problem.type, 'amplitude')
    
    PsiAmplitude     = log(Psi);
    outputSupport    = input_data.problem.constraint.support;
    
    outputPsi = zeros(size(Psi));
    
    outputPsi(outputSupport) = PsiAmplitude(outputSupport);
    
    outputPsi = exp(outputPsi);
        
elseif strcmp(input_data.problem.type, 'mixed')          
    
    PsiPhase         = angle(Psi);
    PsiAmplitude     = abs(Psi);
    outputSupport    = input_data.problem.constraint.support;
    
    outputPsiPhase     = zeros(size(Psi));
    outputPsiAmplitude = zeros(size(Psi));
    
    outputPsiPhase(outputSupport) = PsiPhase(outputSupport);
    outputPsiAmplitude(outputSupport) = PsiAmplitude(outputSupport);
    
    outputPsi        = outputPsiAmplitude .* exp(1i*outputPsiPhase);
    
elseif strcmp(input_data.problem.type, 'real')
    
    outputSupport    = input_data.problem.constraint.support;
    outputPsi        = zeros(size(Psi));
    outputPsi(outputSupport) = Psi(outputSupport);
    
else error('No suitable problem type specified in input_data.problem.type.')

end