% Implements support and range constraints for different objects.

function outputPsi = proxSupportRangeConstraint (input_data, Psi)

if strcmp(input_data.problem.type, 'phase')

    PsiPhase         = angle(Psi);
    outputSupport    = input_data.problem.constraint.support;
    
    outputPsiPhase   = zeros(size(Psi));
    
    % if no range type is specified, we go with negativity for phase
    % objects.
    if isfield(input_data.problem.constraint, 'range')
        if strcmp(input_data.problem.constraint.range, 'positive')
            outputPsiPhase(outputSupport) = max(PsiPhase(outputSupport), 0);
        else        
            outputPsiPhase(outputSupport) = min(PsiPhase(outputSupport), 0);
        end
    else
            outputPsiPhase(outputSupport) = min(PsiPhase(outputSupport), 0);
    end
    outputPsi        = exp(1i*outputPsiPhase);
    
elseif strcmp(input_data.problem.type, 'amplitude')
    
    PsiAmplitude     = abs(Psi);
    outputSupport    = input_data.problem.constraint.support;
    
    outputPsiAmplitude = ones(size(Psi));
    outputPsiAmplitude(outputSupport) = min(PsiAmplitude(outputSupport), 1);
            
    outputPsi        = outputPsiAmplitude;

elseif strcmp(input_data.problem.type, 'mixed')          
    
    PsiPhase         = angle(Psi);
    PsiAmplitude     = abs(Psi);
    outputSupport    = input_data.problem.constraint.support;
    
    outputPsiPhase(outputSupport) = min(PsiPhase(outputSupport), 0);
    outputPsiAmplitude(outputSupport) = min(PsiAmplitude(outputSupport), 1);
       
    outputPsi        = outputPsiAmplitude .* exp(1i*outputPsiPhase);
    
elseif strcmp(input_data.problem.type, 'real')
    
    outputSupport    = input_data.problem.constraint.support;
    outputPsi        = zeros(size(Psi));
    outputPsi(outputSupport) = max(real(Psi(outputSupport)), 0);
    
else error('No suitable problem type specified in input_data.problem.type.')

end