function error = computeError( input_data, newIterate, lastIterate, shadow )
%COMPUTEERROR Computes the error using a specified norm given in the
%   input_data structure.
%   Input: input_data - structure containing all problem information.
%           newIterate - x^(k+1)
%           lastIterate - x^(k+1)
%
% Note that for simulated data, input_data.Psi contains the true solution
% and we can compare the real error. Otherwise we need to use some
% different errors like norm(newIterate - lastIterate).

if isfield(input_data, 'data')
    if strcmp(input_data.algorithm.metric, 'shadow')
        error = norm(shadow-input_data.Psi, 'fro');                                                           % ||P_S x^k - x||_F
    elseif strcmp(input_data.algorithm.metric, 'norm')
        error = norm(newIterate-input_data.Psi,'fro');                                                 % ||x^{k+1} - x||_F
    end
else
    if strcmp(input_data.algorithm.metric, 'shadow')
        error = norm(shadow-magnitudeProjection(input_data, lastIterate),'fro');                        % || P_S x^k - P_M x^k ||_F
    elseif strcmp(input_data.algorithm.metric, 'setdistance')
        error = norm(magnitudeProjection(input_data, shadow)-shadow, 'fro')^2/norm(shadow,'fro')^2;     % ||P_M P_S x^ - P_S x^k||^2_F / || P_S x^k||^2_F
    else strcmp(input_data.algorithm.metric, 'norm')
        error = norm(newIterate-lastIterate,'fro');                                                     % || x^k+1 -x^k||_F
    end
end

end

