function beta = computeRelaxationParameter( input_data, k )
%COMPUTERELAXATIONPARAMETER For a given input_data structure and an
%iteration number, a relaxation parameter is computed.
%   Input: input_data structure.
%   Output: Relaxation parameter beta_k for the RAAR algorithm.

if isfield(input_data.algorithm, 'parameter')
    if strcmp(input_data.algorithm.parameter, 'dynamic') 
        if isfield(input_data.algorithm.beta, 'null')
            beta_0      = input_data.algorithm.beta.null;
        else
            beta_0      = 0.99;      % starting relaxation prameter 
        end
        if isfield(input_data.algorithm.beta, 'max')
            beta_max    = input_data.algorithm.beta.max;
        else
            beta_max    = 0.55;
        end
        if isfield(input_data.algorithm.beta, 'switch')
            beta_switch = input_data.algorithm.beta.switch;
        else
            beta_switch = 20;
        end
        beta = exp((-k/beta_switch).^3)*beta_0+(1-exp((-k/beta_switch).^3))*beta_max;
    
    elseif strcmp(input_data.algorithm.parameter, 'const')
    
        if isfield(input_data.algorithm.beta, 'value')
            beta = input_data.algorithm.beta.value;
        else
            beta = 0.55;
        end
    end
else
    beta = 0.55;
end
  
end

