% prop_fresnel_ff(im,Fx,Fy,f) propagates with fresnelnumbers to the 
%
% Propagation based on just the fresnelnumber and not on special values
%
% The Propagation kernel is sampled in real space, which is better for
% propagations to smaller fresnelnumbers (stronger holographic regime)
% Author: Martin Krenkel

function Im = prop_fresnel(im,fresnelx,fresnely,f)


%%% input:
% im: image to be transformed
% fresnelx:
% fresnely: 
% f: 



[Ny, Nx] = size(im); % size of the image 

% Sampling criterion for phase chirp.
px = abs(1/(Nx*fresnelx));
py = abs(1/(Ny*fresnely));


if (nargin < 4 || f == 0)  % if f is not given or f=0 
	f = max(px,py);
end

	
if (max(Nx,Ny)*f > 4000)
	%yn = input(sprintf('Enlarged image would be very large (%ix%i Pixels).\nReally use this factor f=%f? (N) ',round(Nx*f),round(Ny*f),f),'s');
	yn='yes';
    if ~(strcmpi(yn,'yes') || strcmpi(yn,'y'))
		Im = -1;
		return;
	end
end

if f > 1
	fprintf('Probe-FOV is enlarged by factor %3.2f for propagation.\n',f);
	
	
	% enlarge array by these sizes   
	pady = round((ceil(f)*Ny -Ny)/2);
	padx = round((ceil(f)*Nx -Nx)/2);   
	im_l = padarray(im,[pady padx],'replicate');
	
	% new coordinate system
	[Ny_l, Nx_l] = size(im_l);
	dqy = 2*pi/(Ny_l);
	dqx = 2*pi/(Nx_l);	
	

	[Qx,Qy] = meshgrid(((1:Nx_l)-floor(Nx_l/2)-1)*dqx,((1:Ny_l)-floor(Ny_l/2)-1)*dqy);
	% paraxial propagator
	kappa_x = -1/2*ifftshift(Qx.^2);
	kappa_y = -1/2*ifftshift(Qy.^2);
	
	% propagation
	Im_l = fftshift(ifft2(fft2(ifftshift(im_l)).*exp(1i*kappa_x/(2*pi*fresnelx)+1i*kappa_y/(2*pi*fresnely))));
	
	% decrease field of view	
	Im = Im_l(pady+1:end-pady,padx+1:end-padx);
else
	
	dqx = 2*pi/(Nx);
	dqy = 2*pi/(Ny);
	
	[Qx,Qy] = meshgrid(((1:Nx)-floor(Nx/2)-1)*dqx,((1:Ny)-floor(Ny/2)-1)*dqy);
	kappa_x = -1/2*ifftshift(Qx.^2);
	kappa_y = -1/2*ifftshift(Qy.^2);
	
	% propagation
	Im = fftshift(ifft2(fft2(ifftshift(im)).*exp(1i*kappa_x/(2*pi*fresnelx)+1i*kappa_y/(2*pi*fresnely))));
	
end