function propagatedWave = wavePropagator( input_data, Psi, type )
%WAVEPROPAGATOR Computes the wave propagation for a given wave function Psi
%   object and type. If no type is given, a forward transform is 
%   perforemd, if type=-1, an inverse transform is performed. This file can
%   be modified to your taste if you want to use and allow for different
%   propagators.
    if nargin == 2
           type = 1;
    end
    if strcmp(input_data.problem.propagator,'Fresnel')
        propagatedWave = prop_fresnel(Psi, type*input_data.problem.Fx, type*input_data.problem.Fy);
    elseif strcmp(input_data.problem.propagator,'Fourier')
        if type == -1
            propagatedWave = ifft2(ifftshift(Psi));
        else
            propagatedWave = fftshift(fft2(Psi)); 
        end
    else
         error('No propagation method defined in input_data object')
    end

end

